from pathlib import Path
from unittest.mock import ANY, AsyncMock, MagicMock, call, patch

import pytest

from ophyd_async.core import DEFAULT_TIMEOUT, init_devices
from ophyd_async.epics.eiger import Odin, OdinWriter, Writing
from ophyd_async.testing import get_mock_put, set_mock_value

ODIN_DETECTOR_NAME = "odin_detector"

OdinDriverAndWriter = tuple[Odin, OdinWriter]


@pytest.fixture
def odin_driver_and_writer(RE) -> OdinDriverAndWriter:
    with init_devices(mock=True):
        driver = Odin("")
        writer = OdinWriter(MagicMock(), driver)

    # Set meta and capturing pvs high
    set_mock_value(driver.meta_active, "Active")
    set_mock_value(driver.capture_rbv, "Capturing")
    set_mock_value(driver.meta_writing, "Writing")
    return driver, writer


async def test_when_open_called_then_file_correctly_set(
    odin_driver_and_writer: OdinDriverAndWriter, tmp_path: Path
):
    driver, writer = odin_driver_and_writer
    path_info = writer._path_provider.return_value  # type: ignore
    expected_filename = "filename.h5"
    path_info.directory_path = tmp_path
    path_info.filename = expected_filename

    await writer.open(ODIN_DETECTOR_NAME)

    get_mock_put(driver.file_path).assert_called_once_with(str(tmp_path), wait=ANY)
    get_mock_put(driver.file_name).assert_called_once_with(expected_filename, wait=ANY)


async def test_when_open_called_then_all_expected_signals_set(
    odin_driver_and_writer: OdinDriverAndWriter,
):
    driver, writer = odin_driver_and_writer
    await writer.open(ODIN_DETECTOR_NAME)

    get_mock_put(driver.data_type).assert_called_once_with("UInt16", wait=ANY)
    get_mock_put(driver.num_to_capture).assert_called_once_with(0, wait=ANY)

    get_mock_put(driver.capture).assert_called_once_with(Writing.CAPTURE, wait=ANY)


async def test_given_data_shape_set_when_open_called_then_describe_has_correct_shape(
    odin_driver_and_writer: OdinDriverAndWriter,
):
    driver, writer = odin_driver_and_writer
    set_mock_value(driver.image_width, 1024)
    set_mock_value(driver.image_height, 768)
    description = await writer.open(ODIN_DETECTOR_NAME)
    assert description["data"]["shape"] == [1, 768, 1024]


async def test_when_closed_then_data_capture_turned_off(
    odin_driver_and_writer: OdinDriverAndWriter,
):
    driver, writer = odin_driver_and_writer
    await writer.close()
    get_mock_put(driver.capture).assert_called_once_with(Writing.DONE, wait=ANY)


@pytest.mark.asyncio
@patch("ophyd_async.epics.eiger._odin_io.wait_for_value")
@patch("ophyd_async.epics.eiger._odin_io.set_and_wait_for_other_value")
async def test_wait_for_active_before_capture_then_wait_for_writing(
    mock_set_and_wait_for_other_value,
    mock_wait_for_value,
    odin_driver_and_writer,
):
    driver, writer = odin_driver_and_writer

    mock_manager = AsyncMock()
    mock_manager.attach_mock(mock_wait_for_value, "mock_wait_for_value")
    mock_manager.attach_mock(
        mock_set_and_wait_for_other_value, "mock_set_and_wait_for_other_value"
    )

    await writer.open(ODIN_DETECTOR_NAME)

    expected_calls = [
        call.mock_wait_for_value(driver.meta_active, "Active", timeout=DEFAULT_TIMEOUT),
        call.mock_set_and_wait_for_other_value(
            driver.capture,
            Writing.CAPTURE,
            driver.capture_rbv,
            "Capturing",
            set_timeout=None,
            wait_for_set_completion=False,
        ),
        call.mock_wait_for_value(
            driver.meta_writing, "Writing", timeout=DEFAULT_TIMEOUT
        ),
    ]

    assert mock_manager.mock_calls == expected_calls



import itertools
import os

import pytest

from ophyd_async.core import StaticPathProvider, TriggerInfo
from ophyd_async.epics import (
    adandor,
    adaravis,
    adcore,
    adkinetix,
    adpilatus,
    adsimdetector,
    advimba,
)

DETECTOR_CLASSES: list[type[adcore.AreaDetector]] = [
    adsimdetector.SimDetector,
    advimba.VimbaDetector,
    adandor.Andor2Detector,
    adaravis.AravisDetector,
    adkinetix.KinetixDetector,
    adpilatus.PilatusDetector,
    adcore.ContAcqAreaDetector,
]

WRITER_CLASSES: list[type[adcore.ADWriter]] = [
    adcore.ADHDFWriter,
    adcore.ADTIFFWriter,
    adcore.ADJPEGWriter,
]

pytestmark = pytest.mark.parametrize(
    "detector_cls, writer_cls",
    list(itertools.product(DETECTOR_CLASSES, WRITER_CLASSES)),
)


async def test_hints_from_hdf_writer(
    ad_standard_det_factory,
    detector_cls: type[adcore.AreaDetector],
    writer_cls: type[adcore.ADWriter],
):
    test_det = ad_standard_det_factory(detector_cls, writer_cls=writer_cls)
    assert test_det.hints == {"fields": [test_det.name]}


async def test_can_read(
    ad_standard_det_factory,
    detector_cls: type[adcore.AreaDetector],
    writer_cls: type[adcore.ADWriter],
):
    # Standard detector can be used as Readable
    test_det = ad_standard_det_factory(detector_cls, writer_cls=writer_cls)
    assert (await test_det.read()) == {}


async def test_describes_writer_dataset(
    ad_standard_det_factory,
    detector_cls: type[adcore.AreaDetector],
    writer_cls: type[adcore.ADWriter],
    one_shot_trigger_info: TriggerInfo,
):
    test_det = ad_standard_det_factory(detector_cls, writer_cls=writer_cls)
    assert await test_det.describe() == {}
    await test_det.stage()
    await test_det.prepare(one_shot_trigger_info)
    assert await test_det.describe() == {
        f"{test_det.name}": {
            "source": (
                f"mock+ca://{detector_cls.__name__[: -len('Detector')].upper()}"
                f"1:{writer_cls.default_suffix}FullFileName_RBV"
            ),
            "shape": [one_shot_trigger_info.exposures_per_event, 10, 10],
            "dtype": "array",
            "dtype_numpy": "<u2",
            "external": "STREAM:",
        }
    }


async def test_can_collect(
    ad_standard_det_factory,
    detector_cls: type[adcore.AreaDetector],
    writer_cls: type[adcore.ADWriter],
    static_path_provider: StaticPathProvider,
    one_shot_trigger_info: TriggerInfo,
):
    path_info = static_path_provider()

    # If we are using the HDF writer, the uri will reference
    uri = str(path_info.directory_path) + "/"
    if writer_cls == adcore.ADHDFWriter:
        uri = uri.rstrip("/") + os.sep + f"{path_info.filename}.h5"
    test_det = ad_standard_det_factory(detector_cls, writer_cls=writer_cls)

    await test_det.stage()
    await test_det.prepare(one_shot_trigger_info)
    docs = [(name, doc) async for name, doc in test_det.collect_asset_docs(1)]
    assert len(docs) == 2
    assert docs[0][0] == "stream_resource"
    stream_resource = docs[0][1]
    sr_uid = stream_resource["uid"]
    assert stream_resource["data_key"] == test_det.name
    assert stream_resource["uri"] == "/".join(
        ["file://localhost", str(uri).lstrip("/")]
    )

    # Construct expected stream resource parameters based on writer
    expected_sres_params = {
        "chunk_shape": (1, 10, 10),
    }
    if writer_cls == adcore.ADHDFWriter:
        expected_sres_params["dataset"] = "/entry/data/data"
    else:
        expected_sres_params["template"] = (
            "ophyd_async_tests_{:06d}" + test_det._writer._file_extension
        )

    assert stream_resource["parameters"] == expected_sres_params
    assert docs[1][0] == "stream_datum"
    stream_datum = docs[1][1]
    assert stream_datum["stream_resource"] == sr_uid
    assert stream_datum["seq_nums"] == {"start": 0, "stop": 0}
    assert stream_datum["indices"] == {"start": 0, "stop": 1}


async def test_can_decribe_collect(
    ad_standard_det_factory,
    detector_cls: type[adcore.AreaDetector],
    writer_cls: type[adcore.ADWriter],
    one_shot_trigger_info: TriggerInfo,
):
    test_det = ad_standard_det_factory(detector_cls, writer_cls=writer_cls)

    assert (await test_det.describe_collect()) == {}
    await test_det.stage()
    await test_det.prepare(one_shot_trigger_info)
    assert (await test_det.describe_collect()) == {
        f"{test_det.name}": {
            "source": (
                f"mock+ca://{detector_cls.__name__[: -len('Detector')].upper()}"
                f"1:{writer_cls.default_suffix}FullFileName_RBV"
            ),
            "shape": [one_shot_trigger_info.exposures_per_event, 10, 10],
            "dtype": "array",
            "dtype_numpy": "<u2",
            "external": "STREAM:",
        }
    }



from collections.abc import Awaitable, Callable
from typing import Any, Generic, get_type_hints, is_typeddict

from ._derived_signal_backend import (
    DerivedSignalBackend,
    SignalTransformer,
    Transform,
    TransformT,
)
from ._device import Device
from ._signal import SignalR, SignalRW, SignalT, SignalW
from ._signal_backend import SignalDatatypeT


class DerivedSignalFactory(Generic[TransformT]):
    """A factory that makes Derived Signals using a many-many Transform.

    :param transform_cls:
        The Transform subclass that will be filled in with parameters in order to
        transform raw to derived and derived to raw.
    :param set_derived:
        An optional async function that takes the output of
        `transform_cls.raw_to_derived` and applies it to the raw devices.
    :param raw_and_transform_devices:
        Devices whose values will be passed as parameters to the `transform_cls`,
        and as arguments to `transform_cls.raw_to_derived`.
    """

    def __init__(
        self,
        transform_cls: type[TransformT],
        set_derived: Callable[..., Awaitable[None]] | None = None,
        **raw_and_transform_devices,
    ):
        self._set_derived = set_derived
        # Check the raw and transform devices match the input arguments of the Transform
        if transform_cls is not Transform:
            expected = list(transform_cls.model_fields) + [
                x
                for x in get_type_hints(transform_cls.raw_to_derived)
                if x not in ["self", "return"]
            ]
            if set(expected) != set(raw_and_transform_devices):
                msg = (
                    f"Expected devices to be passed as keyword arguments {expected}, "
                    f"got {list(raw_and_transform_devices)}"
                )
                raise TypeError(msg)
        self._set_derived_takes_dict = (
            is_typeddict(_get_first_arg_datatype(set_derived)) if set_derived else False
        )
        self._transformer = SignalTransformer(
            transform_cls,
            set_derived,
            self._set_derived_takes_dict,
            **raw_and_transform_devices,
        )

    def _make_signal(
        self,
        signal_cls: type[SignalT],
        datatype: type[SignalDatatypeT],
        name: str,
        units: str | None = None,
        precision: int | None = None,
    ) -> SignalT:
        # Check up front the raw_devices are of the right type for what the signal_cls
        # supports
        if issubclass(signal_cls, SignalR):
            self._transformer.raw_and_transform_readables  # noqa: B018
            self._transformer.raw_and_transform_subscribables  # noqa: B018
        if issubclass(signal_cls, SignalW) and not self._set_derived:
            msg = (
                f"Must define a set_derived method to support derived "
                f"{signal_cls.__name__}s"
            )
            raise ValueError(msg)
        if issubclass(signal_cls, SignalRW) and self._set_derived_takes_dict:
            self._transformer.raw_locatables  # noqa: B018
        backend = DerivedSignalBackend(
            datatype, name, self._transformer, units, precision
        )
        return signal_cls(backend)

    def derived_signal_r(
        self,
        datatype: type[SignalDatatypeT],
        name: str,
        units: str | None = None,
        precision: int | None = None,
    ) -> SignalR[SignalDatatypeT]:
        """Create a read only derived signal.

        :param datatype: The datatype of the derived signal value
        :param name:
            The name of the derived signal. Should be a key within the
            return value of `transform_cls.raw_to_derived()`.
        :param units: Engineering units for the derived signal
        :param precision: Number of digits after the decimal place to display
        """
        return self._make_signal(SignalR, datatype, name, units, precision)

    def derived_signal_w(
        self,
        datatype: type[SignalDatatypeT],
        name: str,
        units: str | None = None,
        precision: int | None = None,
    ) -> SignalW[SignalDatatypeT]:
        """Create a write only derived signal.

        :param datatype: The datatype of the derived signal value
        :param name:
            The name of the derived signal. Should be a key within the
            return value of `transform_cls.raw_to_derived()`.
        :param units: Engineering units for the derived signal
        :param precision: Number of digits after the decimal place to display
        """
        return self._make_signal(SignalW, datatype, name, units, precision)

    def derived_signal_rw(
        self,
        datatype: type[SignalDatatypeT],
        name: str,
        units: str | None = None,
        precision: int | None = None,
    ) -> SignalRW[SignalDatatypeT]:
        """Create a read-write derived signal.

        :param datatype: The datatype of the derived signal value
        :param name:
            The name of the derived signal. Should be a key within the
            return value of `transform_cls.raw_to_derived()`.
        :param units: Engineering units for the derived signal
        :param precision: Number of digits after the decimal place to display
        """
        return self._make_signal(SignalRW, datatype, name, units, precision)

    async def transform(self) -> TransformT:
        """Return an instance of `transform_cls` with all the parameters filled in."""
        return await self._transformer.get_transform()


def _get_return_datatype(func: Callable[..., SignalDatatypeT]) -> type[SignalDatatypeT]:
    args = get_type_hints(func)
    if "return" not in args:
        msg = f"{func} does not have a type hint for it's return value"
        raise TypeError(msg)
    return args["return"]


def _get_first_arg_datatype(
    func: Callable[[SignalDatatypeT], Any],
) -> type[SignalDatatypeT]:
    args = get_type_hints(func)
    args.pop("return", None)
    if not args:
        msg = f"{func} does not have a type hinted argument"
        raise TypeError(msg)
    return list(args.values())[0]


def _make_factory(
    raw_to_derived: Callable[..., SignalDatatypeT] | None = None,
    set_derived: Callable[[SignalDatatypeT], Awaitable[None]] | None = None,
    raw_devices: dict[str, Device] | None = None,
) -> DerivedSignalFactory:
    if raw_to_derived:

        class DerivedTransform(Transform):
            def raw_to_derived(self, **kwargs) -> dict[str, SignalDatatypeT]:
                return {"value": raw_to_derived(**kwargs)}

        # Update the signature for raw_to_derived to match what we are passed as this
        # will be checked in DerivedSignalFactory
        DerivedTransform.raw_to_derived.__annotations__ = get_type_hints(raw_to_derived)

        return DerivedSignalFactory(
            DerivedTransform, set_derived=set_derived, **(raw_devices or {})
        )
    else:
        return DerivedSignalFactory(Transform, set_derived=set_derived)


def derived_signal_r(
    raw_to_derived: Callable[..., SignalDatatypeT],
    derived_units: str | None = None,
    derived_precision: int | None = None,
    **raw_devices: Device,
) -> SignalR[SignalDatatypeT]:
    """Create a read only derived signal.

    :param raw_to_derived:
        A function that takes the raw values as individual keyword arguments and
        returns the derived value.
    :param derived_units: Engineering units for the derived signal
    :param derived_precision: Number of digits after the decimal place to display
    :param raw_devices:
        A dictionary of Devices to provide the values for raw_to_derived. The names
        of these arguments must match the arguments of raw_to_derived.
    """
    factory = _make_factory(raw_to_derived=raw_to_derived, raw_devices=raw_devices)
    return factory.derived_signal_r(
        datatype=_get_return_datatype(raw_to_derived),
        name="value",
        units=derived_units,
        precision=derived_precision,
    )


def derived_signal_rw(
    raw_to_derived: Callable[..., SignalDatatypeT],
    set_derived: Callable[[SignalDatatypeT], Awaitable[None]],
    derived_units: str | None = None,
    derived_precision: int | None = None,
    **raw_devices: Device,
) -> SignalRW[SignalDatatypeT]:
    """Create a read-write derived signal.

    :param raw_to_derived:
        A function that takes the raw values as individual keyword arguments and
        returns the derived value.
    :param set_derived:
        A function that takes the derived value and sets the raw signals. It can
        either be an async function, or return an [](#AsyncStatus)
    :param derived_units: Engineering units for the derived signal
    :param derived_precision: Number of digits after the decimal place to display
    :param raw_devices:
        A dictionary of Devices to provide the values for raw_to_derived. The names
        of these arguments must match the arguments of raw_to_derived.
    """
    raw_to_derived_datatype = _get_return_datatype(raw_to_derived)
    set_derived_datatype = _get_first_arg_datatype(set_derived)
    if raw_to_derived_datatype != set_derived_datatype:
        msg = (
            f"{raw_to_derived} has datatype {raw_to_derived_datatype} "
            f"!= {set_derived_datatype} dataype {set_derived_datatype}"
        )
        raise TypeError(msg)

    factory = _make_factory(
        raw_to_derived=raw_to_derived, set_derived=set_derived, raw_devices=raw_devices
    )
    return factory.derived_signal_rw(
        datatype=raw_to_derived_datatype,
        name="value",
        units=derived_units,
        precision=derived_precision,
    )


def derived_signal_w(
    set_derived: Callable[[SignalDatatypeT], Awaitable[None]],
    derived_units: str | None = None,
    derived_precision: int | None = None,
) -> SignalW[SignalDatatypeT]:
    """Create a write only derived signal.

    :param set_derived:
        A function that takes the derived value and sets the raw signals. It can
        either be an async function, or return an [](#AsyncStatus)
    :param derived_units: Engineering units for the derived signal
    :param derived_precision: Number of digits after the decimal place to display
    """
    factory = _make_factory(set_derived=set_derived)
    return factory.derived_signal_w(
        datatype=_get_first_arg_datatype(set_derived),
        name="value",
        units=derived_units,
        precision=derived_precision,
    )



from unittest.mock import ANY, AsyncMock, MagicMock

import pytest

from ophyd_async.core import DetectorTrigger, init_devices
from ophyd_async.fastcs.eiger import EigerDetector, EigerTriggerInfo
from ophyd_async.testing import get_mock_put, set_mock_value


@pytest.fixture
def detector(RE):
    with init_devices(mock=True):
        detector = EigerDetector("BL03I", MagicMock())
    set_mock_value(detector.odin.meta_active, "Active")
    set_mock_value(detector.odin.capture_rbv, "Capturing")
    set_mock_value(detector.odin.meta_writing, "Writing")
    return detector


async def test_when_prepared_with_energy_then_energy_set_on_detector(detector):
    detector._controller.arm = AsyncMock()
    await detector.prepare(
        EigerTriggerInfo(
            exposure_timeout=None,
            number_of_events=1,
            trigger=DetectorTrigger.INTERNAL,
            energy_ev=10000,
        )
    )

    get_mock_put(detector.drv.detector.photon_energy).assert_called_once_with(
        10000, wait=ANY
    )



from ophyd_async.core import StrictEnum, SubsetEnum
from ophyd_async.epics.adcore import ADBaseIO
from ophyd_async.epics.core import (
    epics_signal_r,
    epics_signal_rw,
)


class Andor2TriggerMode(StrictEnum):
    INTERNAL = "Internal"
    EXT_TRIGGER = "External"
    EXT_START = "External Start"
    EXT_EXPOSURE = "External Exposure"
    EXT_FVP = "External FVP"
    SOFTWARE = "Software"


class Andor2DataType(SubsetEnum):
    UINT16 = "UInt16"
    UINT32 = "UInt32"
    FLOAT32 = "Float32"
    FLOAT64 = "Float64"


class Andor2DriverIO(ADBaseIO):
    """Epics pv for andor model:DU897_BV as deployed on p99."""

    def __init__(self, prefix: str, name: str = "") -> None:
        super().__init__(prefix, name=name)
        self.trigger_mode = epics_signal_rw(Andor2TriggerMode, prefix + "TriggerMode")
        self.data_type = epics_signal_r(Andor2DataType, prefix + "DataType_RBV")  # type: ignore
        self.andor_accumulate_period = epics_signal_r(
            float, prefix + "AndorAccumulatePeriod_RBV"
        )



from ._andor import Andor2Detector
from ._andor_controller import Andor2Controller
from ._andor_io import Andor2DriverIO

__all__ = [
    "Andor2Detector",
    "Andor2Controller",
    "Andor2DriverIO",
]



import asyncio
import re
from unittest.mock import call

import pytest

from ophyd_async.core import (
    derived_signal_r,
    derived_signal_rw,
    soft_signal_r_and_setter,
    soft_signal_rw,
)
from ophyd_async.testing import (
    BeamstopPosition,
    Exploder,
    MovableBeamstop,
    ReadOnlyBeamstop,
    assert_describe_signal,
    assert_reading,
    assert_value,
    get_mock,
)


@pytest.mark.parametrize(
    "x, y, position",
    [
        (0, 0, BeamstopPosition.IN_POSITION),
        (3, 5, BeamstopPosition.OUT_OF_POSITION),
    ],
)
@pytest.mark.parametrize("cls", [ReadOnlyBeamstop, MovableBeamstop])
async def test_get_returns_right_position(
    cls: type[ReadOnlyBeamstop | MovableBeamstop],
    x: float,
    y: float,
    position: BeamstopPosition,
):
    inst = cls("inst")
    await inst.x.set(x)
    await inst.y.set(y)
    await assert_value(inst.position, position)
    await assert_reading(inst.position, {"inst-position": {"value": position}})
    await assert_describe_signal(
        inst.position,
        choices=[
            "In position",
            "Out of position",
        ],
        dtype="string",
        dtype_numpy="|S40",
        shape=[],
    )


@pytest.mark.parametrize("cls", [ReadOnlyBeamstop, MovableBeamstop])
async def test_monitoring_position(cls: type[ReadOnlyBeamstop | MovableBeamstop]):
    results = asyncio.Queue[BeamstopPosition]()
    inst = cls("inst")
    inst.position.subscribe_value(results.put_nowait)
    assert await results.get() == BeamstopPosition.IN_POSITION
    assert results.empty()
    await inst.x.set(3)
    assert await results.get() == BeamstopPosition.OUT_OF_POSITION
    assert results.empty()
    await inst.y.set(5)
    assert await results.get() == BeamstopPosition.OUT_OF_POSITION
    assert results.empty()
    await asyncio.gather(inst.x.set(0), inst.y.set(0))
    assert await results.get() == BeamstopPosition.OUT_OF_POSITION
    assert await results.get() == BeamstopPosition.IN_POSITION
    assert results.empty()


async def test_setting_position():
    inst = MovableBeamstop("inst")
    # Connect in mock mode so we can see what would have been set
    await inst.connect(mock=True)
    m = get_mock(inst)
    await inst.position.set(BeamstopPosition.OUT_OF_POSITION)
    assert m.mock_calls == [
        call.position.put(BeamstopPosition.OUT_OF_POSITION, wait=True),
        call.x.put(3, wait=True),
        call.y.put(5, wait=True),
    ]
    m.reset_mock()
    await inst.position.set(BeamstopPosition.IN_POSITION)
    assert m.mock_calls == [
        call.position.put(BeamstopPosition.IN_POSITION, wait=True),
        call.x.put(0, wait=True),
        call.y.put(0, wait=True),
    ]


async def test_setting_all():
    inst = Exploder(3, "exploder")
    await assert_reading(
        inst, {f"exploder-signals-{i}": {"value": 0} for i in range(1, 4)}
    )
    await inst.set_all.set(5)
    await assert_reading(
        inst, {f"exploder-signals-{i}": {"value": 5} for i in range(1, 4)}
    )


def test_mismatching_args():
    def _get_position(x: float, y: float) -> BeamstopPosition:
        if abs(x) < 1 and abs(y) < 2:
            return BeamstopPosition.IN_POSITION
        else:
            return BeamstopPosition.OUT_OF_POSITION

    with pytest.raises(
        TypeError,
        match=re.escape(
            "Expected devices to be passed as keyword arguments ['x', 'y'], "
            "got ['foo', 'bar']"
        ),
    ):
        derived_signal_r(
            _get_position, foo=soft_signal_rw(float), bar=soft_signal_rw(float)
        )


async def test_derived_signal_rw_works_with_signal_r():
    signal_r, _ = soft_signal_r_and_setter(int, initial_value=4)

    def _get(ts: int) -> float:
        return ts

    async def _put(value: float) -> None:
        pass

    derived = derived_signal_rw(_get, _put, ts=signal_r)
    assert await derived.get_value() == 4



from unittest.mock import AsyncMock, patch

import pytest

from ophyd_async.core import (
    DetectorTrigger,
    TriggerInfo,
)
from ophyd_async.epics import adaravis


@pytest.fixture
def test_adaravis(ad_standard_det_factory) -> adaravis.AravisDetector:
    return ad_standard_det_factory(adaravis.AravisDetector)


@pytest.mark.parametrize("exposure_time", [0.0, 0.1, 1.0, 10.0, 100.0])
async def test_deadtime_invariant_with_exposure_time(
    exposure_time: float,
    test_adaravis: adaravis.AravisDetector,
):
    assert test_adaravis._controller.get_deadtime(exposure_time) == 1961e-6


async def test_unsupported_trigger_excepts(test_adaravis: adaravis.AravisDetector):
    with patch(
        "ophyd_async.epics.adcore._hdf_writer.ADHDFWriter.open", new_callable=AsyncMock
    ) as mock_open:
        with pytest.raises(
            ValueError,
            # str(EnumClass.value) handling changed in Python 3.11
            match="ADAravis does not support (DetectorTrigger.)?VARIABLE_GATE",
        ):
            await test_adaravis.prepare(
                TriggerInfo(
                    number_of_events=0,
                    trigger=DetectorTrigger.VARIABLE_GATE,
                    deadtime=1,
                    livetime=1,
                    exposure_timeout=3,
                )
            )

    mock_open.assert_called_once()
